from typing import cast

from saic_ismart_client.common_model import Asn1Type, ApplicationData

FIELD_VALUE = 'value'
FIELD_ID = 'id'
FIELD_VEHICLE_ALERTS = 'vehicleAlerts'
FIELD_SECONDS = 'seconds'
FIELD_ALTITUDE = 'altitude'
FIELD_LONGITUDE = 'longitude'
FIELD_LATITUDE = 'latitude'
FIELD_SATELLITES = 'satellites'
FIELD_HDOP = 'hdop'
FIELD_SPEED = 'speed'
FIELD_HEADING = 'heading'
FIELD_POSITION = 'position'
FIELD_GPS_STATUS = 'gpsStatus'
FIELD_TIMESTAMP_4_SHORT = 'timestamp4Short'
FIELD_WAY_POINT = 'wayPoint'
FIELD_EXTENDED_VEHICLE_STATUS = 'extendedVehicleStatus'
FIELD_BASIC_VEHICLE_STATUS = 'basicVehicleStatus'
FIELD_GPS_POSITION = 'gpsPosition'
FIELD_STATUS_TIME = 'statusTime'
FIELD_PARAM_VALUE = 'paramValue'
FIELD_PARAM_ID = 'paramId'
FIELD_RVC_PARAMS = 'rvcParams'
FIELD_RVC_REQ_TYPE = 'rvcReqType'
FIELD_VEH_STATUS_REQ_TYPE = 'vehStatusReqType'


class OtaRvmVehicleStatusReq(ApplicationData):
    def __init__(self):
        super().__init__('OTARVMVehicleStatusReq')
        self.veh_status_req_type = None

    def get_data(self) -> dict:
        return {FIELD_VEH_STATUS_REQ_TYPE: self.veh_status_req_type}

    def init_from_dict(self, data: dict):
        self.veh_status_req_type = data.get(FIELD_VEH_STATUS_REQ_TYPE)


class RvsWgs84Point(Asn1Type):
    def __init__(self):
        super().__init__('RvsWGS84Point')
        self.latitude = None
        self.longitude = None
        self.altitude = None

    def get_data(self) -> dict:
        return {
            FIELD_LATITUDE: self.latitude,
            FIELD_LONGITUDE: self.longitude,
            FIELD_ALTITUDE: self.altitude
        }

    def init_from_dict(self, data: dict):
        self.latitude = data.get(FIELD_LATITUDE)
        self.longitude = data.get(FIELD_LONGITUDE)
        self.altitude = data.get(FIELD_ALTITUDE)


class RvsWayPoint(Asn1Type):
    def __init__(self):
        super().__init__('RvsWayPoint')
        self.position = None
        self.heading = None
        self.speed = None
        self.hdop = None
        self.satellites = None

    def get_data(self) -> dict:
        return {
            FIELD_POSITION: self.position.get_data(),
            FIELD_HEADING: self.heading,
            FIELD_SPEED: self.speed,
            FIELD_HDOP: self.hdop,
            FIELD_SATELLITES: self.satellites
        }

    def init_from_dict(self, data: dict):
        self.position = RvsWgs84Point()
        self.position.init_from_dict(data.get(FIELD_POSITION))
        self.heading = data.get(FIELD_HEADING)
        self.speed = data.get(FIELD_SPEED)
        self.hdop = data.get(FIELD_HDOP)
        self.satellites = data.get(FIELD_SATELLITES)

    def get_position(self) -> RvsWgs84Point:
        return cast(RvsWgs84Point, self.position)


class RvsPosition(Asn1Type):
    def __init__(self):
        super().__init__('RvsPosition')
        self.way_point = None
        self.timestamp_4_short = None
        self.gps_status = None

    def get_data(self) -> dict:
        return {
            FIELD_WAY_POINT: self.way_point.get_data(),
            FIELD_TIMESTAMP_4_SHORT: self.timestamp_4_short.get_data(),
            FIELD_GPS_STATUS: self.gps_status
        }

    def init_from_dict(self, data: dict):
        self.way_point = RvsWayPoint()
        self.way_point.init_from_dict(data.get(FIELD_WAY_POINT))
        self.timestamp_4_short = Timestamp4Short()
        self.timestamp_4_short.init_from_dict(data.get(FIELD_TIMESTAMP_4_SHORT))
        self.gps_status = data.get(FIELD_GPS_STATUS)

    def get_way_point(self) -> RvsWayPoint:
        return cast(RvsWayPoint, self.way_point)


class Timestamp4Short(Asn1Type):
    def __init__(self):
        super().__init__('Timestamp4Short')
        self.seconds = None
        
    def get_data(self) -> dict:
        return {
            FIELD_SECONDS: self.seconds
        }
    
    def init_from_dict(self, data: dict):
        self.seconds = data.get(FIELD_SECONDS)
        

class RvsBasicStatus25857(Asn1Type):
    def __init__(self):
        super().__init__('RvsBasicStatus25857')
        self.driver_door = None  # BOOLEAN
        self.passenger_door = None  # BOOLEAN
        self.rear_left_door = None  # BOOLEAN
        self.rear_right_door = None  # BOOLEAN
        self.boot_status = None  # BOOLEAN
        self.bonnet_status = None  # BOOLEAN
        self.lock_status = None  # BOOLEAN
        self.driver_window = None  # BOOLEAN OPTIONAL
        self.passenger_window = None  # BOOLEAN OPTIONAL,
        self.rear_left_window = None  # BOOLEAN OPTIONAL,
        self.rear_right_window = None  # BOOLEAN OPTIONAL,
        self.sun_roof_status = None  # BOOLEAN OPTIONAL,
        self.front_right_tyre_pressure = None  # INTEGER(0..255) OPTIONAL,
        self.front_left_tyre_pressure = None  # INTEGER(0..255) OPTIONAL,
        self.rear_right_tyre_pressure = None  # INTEGER(0..255) OPTIONAL,
        self.rear_left_tyre_pressure = None  # INTEGER(0..255) OPTIONAL,
        self.wheel_tyre_monitor_status = None  # INTEGER(0..255) OPTIONAL,
        self.side_light_status = None  # BOOLEAN,
        self.dipped_beam_status = None  # BOOLEAN,
        self.main_beam_status = None  # BOOLEAN,
        self.vehicle_alarm_status = None  # INTEGER(0..255) OPTIONAL,
        self.engine_status = None  # INTEGER(0..255),
        self.power_mode = None  # INTEGER(0..255),
        self.last_key_seen = None  # INTEGER(0..65535),
        self.current_journey_distance = None  # INTEGER(0..65535),
        self.current_journey_id = None  # INTEGER(0..2147483647),
        self.interior_temperature = None  # INTEGER(-128..127),
        self.exterior_temperature = None  # INTEGER(-128..127),
        self.fuel_level_prc = None  # INTEGER(0..255),
        self.fuel_range = None  # INTEGER(0..65535),
        self.remote_climate_status = None  # INTEGER(0..255),
        self.front_left_seat_heat_level = None  # INTEGER(0..255) OPTIONAL,
        self.front_right_seat_heat_level = None  # INTEGER(0..255) OPTIONAL,
        self.can_bus_active = None  # BOOLEAN,
        self.time_of_last_canbus_activity = None  # INTEGER(0..2147483647),
        self.clstr_dspd_fuel_lvl_sgmt = None  # INTEGER(0..255),
        self.mileage = None  # INTEGER(0..2147483647),
        self.battery_voltage = None  # INTEGER(0..65535),
        self.hand_brake = None  # BOOLEAN,
        self.veh_elec_rng_dsp = None  # INTEGER(0..255),
        self.fuel_range_elec = None  # INTEGER(0..65535) OPTIONAL,
        self.rmt_htd_rr_wnd_st = None  # INTEGER(0..255),
        self.extended_data1 = None  # INTEGER(0..2147483647) OPTIONAL,
        self.extended_data2 = None  # INTEGER(0..2147483647) OPTIONAL

    def get_data(self) -> dict:
        data = {
            'driverDoor': self.driver_door,
            'passengerDoor': self.passenger_door,
            'rearLeftDoor': self.rear_left_door,
            'rearRightDoor': self.rear_right_door,
            'bootStatus': self.boot_status,
            'bonnetStatus': self.bonnet_status,
            'lockStatus': self.lock_status,
            'sideLightStatus': self.side_light_status,
            'dippedBeamStatus': self.dipped_beam_status,
            'mainBeamStatus': self.main_beam_status,
            'engineStatus': self.engine_status,
            'powerMode': self.power_mode,
            'lastKeySeen': self.last_key_seen,
            'currentjourneyDistance': self.current_journey_distance,
            'currentJourneyID': self.current_journey_id,
            'interiorTemperature': self.interior_temperature,
            'exteriorTemperature': self.exterior_temperature,
            'fuelLevelPrc': self.fuel_level_prc,
            'fuelRange': self.fuel_range,
            'remoteClimateStatus': self.remote_climate_status,
            'canBusActive': self.can_bus_active,
            'timeOfLastCANBUSActivity': self.time_of_last_canbus_activity,
            'clstrDspdFuelLvlSgmt': self.clstr_dspd_fuel_lvl_sgmt,
            'mileage': self.mileage,
            'batteryVoltage': self.battery_voltage,
            'handBrake': self.hand_brake,
            'vehElecRngDsp': self.veh_elec_rng_dsp,
            'rmtHtdRrWndSt': self.rmt_htd_rr_wnd_st
        }
        self.add_optional_field_to_data(data, 'driverWindow', self.driver_window)
        self.add_optional_field_to_data(data, 'passengerWindow', self.passenger_window)
        self.add_optional_field_to_data(data, 'rearLeftWindow', self.rear_left_window)
        self.add_optional_field_to_data(data, 'rearRightWindow', self.rear_right_window)
        self.add_optional_field_to_data(data, 'sunroofStatus', self.sun_roof_status)
        self.add_optional_field_to_data(data, 'frontRrightTyrePressure', self.front_right_tyre_pressure)
        self.add_optional_field_to_data(data, 'frontLeftTyrePressure', self.front_left_tyre_pressure)
        self.add_optional_field_to_data(data, 'rearRightTyrePressure', self.rear_right_tyre_pressure)
        self.add_optional_field_to_data(data, 'rearLeftTyrePressure', self.rear_left_tyre_pressure)
        self.add_optional_field_to_data(data, 'wheelTyreMonitorStatus', self.wheel_tyre_monitor_status)
        self.add_optional_field_to_data(data, 'vehicleAlarmStatus', self.vehicle_alarm_status)
        self.add_optional_field_to_data(data, 'frontLeftSeatHeatLevel', self.front_left_seat_heat_level)
        self.add_optional_field_to_data(data, 'frontRightSeatHeatLevel', self.front_right_seat_heat_level)
        self.add_optional_field_to_data(data, 'fuelRangeElec', self.fuel_range_elec)
        self.add_optional_field_to_data(data, 'extendedData1', self.extended_data1)
        self.add_optional_field_to_data(data, 'extendedData2', self.extended_data2)
        return data

    def init_from_dict(self, data: dict):
        self.driver_door = data.get('driverDoor')
        self.passenger_door = data.get('passengerDoor')
        self.rear_left_door = data.get('rearLeftDoor')
        self.rear_right_door = data.get('rearRightDoor')
        self.boot_status = data.get('bootStatus')
        self.bonnet_status = data.get('bonnetStatus')
        self.lock_status = data.get('lockStatus')
        self.side_light_status = data.get('sideLightStatus')
        self.dipped_beam_status = data.get('dippedBeamStatus')
        self.main_beam_status = data.get('mainBeamStatus')
        self.engine_status = data.get('engineStatus')
        self.power_mode = data.get('powerMode')
        self.last_key_seen = data.get('lastKeySeen')
        self.current_journey_distance = data.get('currentjourneyDistance')
        self.current_journey_id = data.get('currentJourneyID')
        self.interior_temperature = data.get('interiorTemperature')
        self.exterior_temperature = data.get('exteriorTemperature')
        self.fuel_level_prc = data.get('fuelLevelPrc')
        self.fuel_range = data.get('fuelRange')
        self.remote_climate_status = data.get('remoteClimateStatus')
        self.can_bus_active = data.get('canBusActive')
        self.time_of_last_canbus_activity = data.get('timeOfLastCANBUSActivity')
        self.clstr_dspd_fuel_lvl_sgmt = data.get('clstrDspdFuelLvlSgmt')
        self.mileage = data.get('mileage')
        self.battery_voltage = data.get('batteryVoltage')
        self.hand_brake = data.get('handBrake')
        self.veh_elec_rng_dsp = data.get('vehElecRngDsp')
        self.rmt_htd_rr_wnd_st = data.get('rmtHtdRrWndSt')
        self.driver_window = data.get('driverWindow')
        self.passenger_window = data.get('passengerWindow')
        self.rear_left_window = data.get('rearLeftWindow')
        self.rear_right_window = data.get('rearRightWindow')
        self.sun_roof_status = data.get('sunroofStatus')
        self.front_right_tyre_pressure = data.get('frontRrightTyrePressure')
        self.front_left_tyre_pressure = data.get('frontLeftTyrePressure')
        self.rear_right_tyre_pressure = data.get('rearRightTyrePressure')
        self.rear_left_tyre_pressure = data.get('rearLeftTyrePressure')
        self.wheel_tyre_monitor_status = data.get('wheelTyreMonitorStatus')
        self.vehicle_alarm_status = data.get('vehicleAlarmStatus')
        self.front_left_seat_heat_level = data.get('frontLeftSeatHeatLevel')
        self.front_right_seat_heat_level = data.get('frontRightSeatHeatLevel')
        self.fuel_range_elec = data.get('fuelRangeElec')
        self.extended_data1 = data.get('extendedData1')
        self.extended_data2 = data.get('extendedData2')

    def extended_data_2_present(self) -> bool:
        return self.extended_data2 is not None


class RvsExtStatus(Asn1Type):
    def __init__(self):
        super().__init__('RvsExtStatus')
        self.vehicle_alerts = []

    def get_data(self) -> dict:
        vehicle_alerts = []
        for alert in self.vehicle_alerts:
            vehicle_alerts.append(alert.get_data())
        return {
            FIELD_VEHICLE_ALERTS: vehicle_alerts
        }

    def init_from_dict(self, data: dict):
        vehicle_alerts = data.get(FIELD_VEHICLE_ALERTS)
        for alert in vehicle_alerts:
            a = VehicleAlertInfo()
            a.init_from_dict(alert)
            self.vehicle_alerts.append(a)


class VehicleAlertInfo(Asn1Type):
    def __init__(self):
        super().__init__('VehicleAlertInfo')
        self.id = None
        self.value = None

    def get_data(self) -> dict:
        return {
            FIELD_ID: self.id,
            FIELD_VALUE: self.value
        }

    def init_from_dict(self, data: dict):
        self.id = data.get(FIELD_ID)
        self.value = data.get(FIELD_VALUE)


class OtaRvcReq(ApplicationData):
    def __init__(self):
        super().__init__('OTARVCReq')
        self.rvc_req_type = None
        self.rvc_params = []

    def get_data(self) -> dict:
        data = {
            FIELD_RVC_REQ_TYPE: self.rvc_req_type
        }
        param_list = []
        for param in self.rvc_params:
            param_list.append(param.get_data())
        data[FIELD_RVC_PARAMS] = param_list

        return data

    def init_from_dict(self, data: dict):
        self.rvc_req_type = data.get(FIELD_RVC_REQ_TYPE)
        if FIELD_RVC_PARAMS in data:
            rvc_params_list = data.get(FIELD_RVC_PARAMS)
            for item in rvc_params_list:
                param = RvcReqParam()
                param.init_from_dict(item)
                self.rvc_params.append(param)


class RvcReqParam(Asn1Type):
    def __init__(self):
        super().__init__('RvcReqParam')
        self.param_id = None
        self.param_value = None

    def get_data(self) -> dict:
        return {
            FIELD_PARAM_ID: self.param_id,
            FIELD_PARAM_VALUE: self.param_value
        }

    def init_from_dict(self, data: dict):
        self.param_id = data.get(FIELD_PARAM_ID)
        self.param_value = data.get(FIELD_PARAM_VALUE)


class OtaRvmVehicleStatusResp25857(ApplicationData):
    def __init__(self):
        super().__init__('OTARVMVehicleStatusResp25857')
        self.status_time = None
        self.gps_position = None
        self.basic_vehicle_status = None
        self.extended_vehicle_status = None

    def get_data(self) -> dict:
        if (
                self.status_time is not None
                and self.gps_position is not None
                and self.basic_vehicle_status is not None
        ):
            data = {
                FIELD_STATUS_TIME: self.status_time,
                FIELD_GPS_POSITION: self.gps_position.get_data(),
                FIELD_BASIC_VEHICLE_STATUS: self.basic_vehicle_status.get_data()
            }
            if FIELD_EXTENDED_VEHICLE_STATUS in data:
                data[FIELD_EXTENDED_VEHICLE_STATUS] = self.extended_vehicle_status.get_data()
            return data
        else:
            return {}

    def init_from_dict(self, data: dict):
        self.status_time = data.get(FIELD_STATUS_TIME)
        self.gps_position = RvsPosition()
        self.gps_position.init_from_dict(data.get(FIELD_GPS_POSITION))
        self.basic_vehicle_status = RvsBasicStatus25857()
        self.basic_vehicle_status.init_from_dict(data.get(FIELD_BASIC_VEHICLE_STATUS))
        if FIELD_EXTENDED_VEHICLE_STATUS in data:
            self.extended_vehicle_status = RvsExtStatus()
            self.extended_vehicle_status.init_from_dict(data.get(FIELD_EXTENDED_VEHICLE_STATUS))

    def get_basic_vehicle_status(self) -> RvsBasicStatus25857:
        return cast(RvsBasicStatus25857, self.basic_vehicle_status)

    def get_gps_position(self) -> RvsPosition:
        return cast(RvsPosition, self.gps_position)

    def is_charging(self) -> bool:
        return (
                self.get_basic_vehicle_status().extended_data_2_present()
                and self.get_basic_vehicle_status().extended_data2 >= 1
        )

    def is_parked(self) -> bool:
        return (
                self.get_basic_vehicle_status().engine_status != 1
                or self.get_basic_vehicle_status().hand_brake
        )

    def is_engine_running(self) -> bool:
        return self.get_basic_vehicle_status().engine_status == 1
